#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "dialogselectpoints.h"



MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow) {

    ui->setupUi(this);
    connect(ui->actionExit, SIGNAL(triggered()), this, SLOT(close()));

    directories.resize(4);
    filenames.resize(4);
    isPossibleToExecute.resize(4);
    for(int i = 0; i < isPossibleToExecute.size(); ++i)
        isPossibleToExecute[i] = false;

    QSignalMapper* signalMapper = new QSignalMapper (this);

    connect(ui->pushButtonLoadKinect1, SIGNAL( clicked() ), signalMapper, SLOT(map()));
    connect(ui->pushButtonLoadKinect2, SIGNAL( clicked() ), signalMapper, SLOT(map()));
    connect(ui->pushButtonLoadKinect3, SIGNAL( clicked() ), signalMapper, SLOT(map()));
    connect(ui->pushButtonLoadKinect4, SIGNAL( clicked() ), signalMapper, SLOT(map()));

    signalMapper->setMapping(ui->pushButtonLoadKinect1, 1);
    signalMapper->setMapping(ui->pushButtonLoadKinect2, 2);
    signalMapper->setMapping(ui->pushButtonLoadKinect3, 3);
    signalMapper->setMapping(ui->pushButtonLoadKinect4, 4);

    connect (signalMapper, SIGNAL(mapped(int)), this, SLOT(openDir(int))) ;

    //CLOSE BUTTON
    connect(ui->pushButton_2, SIGNAL( clicked() ), this, SLOT( close() ) );

    //EXECUTE BUTTON

    ui->pushButtonExecuteKinect1->setStyleSheet("color: rgb(120, 120, 120); color: rgb(120, 120, 120)");
    ui->pushButtonExecuteKinect2->setStyleSheet("color: rgb(120, 120, 120); color: rgb(120, 120, 120)");
    ui->pushButtonExecuteKinect3->setStyleSheet("color: rgb(120, 120, 120); color: rgb(120, 120, 120)");
    ui->pushButtonExecuteKinect4->setStyleSheet("color: rgb(120, 120, 120); color: rgb(120, 120, 120)");

    QSignalMapper* signalMapperExecute = new QSignalMapper (this);

    connect(ui->pushButtonExecuteKinect1, SIGNAL( clicked() ), signalMapperExecute, SLOT( map() ) );
    connect(ui->pushButtonExecuteKinect2, SIGNAL( clicked() ), signalMapperExecute, SLOT( map() ) );
    connect(ui->pushButtonExecuteKinect3, SIGNAL( clicked() ), signalMapperExecute, SLOT( map() ) );
    connect(ui->pushButtonExecuteKinect4, SIGNAL( clicked() ), signalMapperExecute, SLOT( map() ) );

    signalMapperExecute->setMapping(ui->pushButtonExecuteKinect1, 1);
    signalMapperExecute->setMapping(ui->pushButtonExecuteKinect2, 2);
    signalMapperExecute->setMapping(ui->pushButtonExecuteKinect3, 3);
    signalMapperExecute->setMapping(ui->pushButtonExecuteKinect4, 4);

    connect (signalMapperExecute, SIGNAL(mapped(int)), this, SLOT(newDialog(int))) ;

}

MainWindow::~MainWindow()
{
    delete ui;
    //delete p;
}

void MainWindow::openDir(int i) {



    std::string dirName;
    if(directories[i-1] == "")
        dirName = "/home";
    else
        dirName = directories[i-1];

    filename = QFileDialog::getOpenFileName(
                this,
                tr("Open Document"),
                dirName.c_str(),
                tr("Document files (*.jpg );;") );

    if(filename.toUtf8().constData() == "" && filenames[i-1] != "") {

    } else
        filenames[i-1] = filename.toUtf8().constData();

    if(filenames[i-1] != "") {
        int index = filenames[i-1].find_last_of("/");

        directories[i-1] = filenames[i-1].substr(0, index);
        isPossibleToExecute[i-1] = true;

        cv::Mat img = cv::imread(filenames[i-1].c_str());
        cv::Mat dst = cv::Mat::zeros(cv::Size(181, 131), img.type());
        cv::resize(img, dst, dst.size());

        QImage *tmpImg = Mat2QImage(dst);
        QPixmap image = QPixmap::fromImage(*tmpImg);

        switch(i) {
            case 1:
                ui->label_kinect_1->setPixmap(image);
                ui->pushButtonExecuteKinect1->setStyleSheet("color: rgb(0, 0, 0); color: rgb(0, 0, 0)");
                break;
            case 2:
                ui->label_kinect_2->setPixmap(image);
                ui->pushButtonExecuteKinect2->setStyleSheet("color: rgb(0, 0, 0); color: rgb(0, 0, 0)");
                break;
            case 3:
                ui->label_kinect_3->setPixmap(image);
                ui->pushButtonExecuteKinect3->setStyleSheet("color: rgb(0, 0, 0); color: rgb(0, 0, 0)");
                break;
            case 4:
                ui->label_kinect_4->setPixmap(image);
                ui->pushButtonExecuteKinect4->setStyleSheet("color: rgb(0, 0, 0); color: rgb(0, 0, 0)");
                break;
        }

        delete tmpImg;
    }
}

void MainWindow::newDialog(int i)
{
    if(isPossibleToExecute[i-1]) {
        DialogSelectPoints win;
        QImage *image = new QImage(":/field.png");
        if(image->isNull()) {
            std::cout << "Error to load the image" << std::endl;
            exit(-1);
        }
        p = new SelectPoint(filenames[i-1], image, i);
        delete image;
        win.setFixedHeight(p->getHeight()+50);
        win.setFixedWidth(p->getWidth()+20);
        win.init(p);
        win.setWindowTitle("Calibration Tool");
        win.exec();
    }
}
