#include "selectpoint.h"
#include "qtcvtransform.h"

SelectPoint::SelectPoint(){

}

SelectPoint::SelectPoint(const std::string objName, const QImage *scene, int num) {

    img_scene = QImage2Mat(scene);
    img_obj = cv::imread(objName.c_str());

    img_complete = cv::Mat::zeros(cv::Size(img_obj.cols + img_scene.cols, img_obj.rows > img_scene.rows ? img_obj.rows : img_scene.rows), CV_8UC3);
    int width = img_obj.cols;
    int height = img_obj.rows;
    for(int i = 0; i < height; ++i) {
        for(int j = 0; j < width; ++j) {
            img_complete.at<cv::Vec3b>(i,j) = img_obj.at<cv::Vec3b>(i,j);
        }
    }
    width = img_obj.cols + img_scene.cols;
    height = img_scene.rows;

    for(int i = 0; i < height ; ++i) {
        for(int j = img_obj.cols, m = 0; j < width && m < img_scene.cols; ++j, ++m) {
            img_complete.at<cv::Vec3b>(i,j) = img_scene.at<cv::Vec3b>(i,m);
        }
    }
    img_complete_clone = img_complete.clone();

    isPossibleToCompute = false;

    std::stringstream file;
    file << "homography" << num << ".txt";

    filename = file.str();
}

SelectPoint::~SelectPoint() {
    delete h;
}

void SelectPoint::changeState(int x, int y) {
    cv::Point2f p;
    bool draw = false;

    if(x <= img_obj.cols && y <= img_obj.rows && obj_points.size() == scene_points.size()) {
        p = cv::Point2f(x,y);
        obj_points.push_back(p);
        dialog->labelMsg->setText("Select Minimum four (4) points and then press Compute!: Select a Point from the Virtual Field");
        dialog->pushButtonCompute->setStyleSheet("color: rgb(120, 120, 120); color: rgb(120, 120, 120)");
        draw = true;
        isPossibleToCompute = false;
    } else if ( x > img_obj.cols && x < img_obj.cols + img_scene.cols && y < img_scene.rows && obj_points.size() != scene_points.size()) {
        p = cv::Point2f(x,y);
        dialog->labelMsg->setText("Select Minimum four (4) points and then press Compute!: Select a Point from the Object View");
        scene_points.push_back(cv::Point2f(p.x - img_obj.cols, p.y));
        draw = true;
        if(obj_points.size() >= 4 && scene_points.size() >= 4) {
            dialog->pushButtonCompute->setStyleSheet("color: rgb(0, 0, 0); color: rgb(0, 0, 0)");
            isPossibleToCompute = true;
        }
    }

    if(draw) {

        if(scene_points.size() != 0 && obj_points.size() == scene_points.size()) {
            cv::line(img_complete_clone, obj_points[obj_points.size()-1], p, CV_RGB(0,0,255), 2);
        }

        cv::line(img_complete_clone, p,	cv::Point2f(p.x+10, p.y), CV_RGB(255,0,0), 2);
        cv::line(img_complete_clone, p,	cv::Point2f(p.x-10, p.y), CV_RGB(255,0,0), 2);
        cv::line(img_complete_clone, p,	cv::Point2f(p.x, p.y+10), CV_RGB(255,0,0), 2);
        cv::line(img_complete_clone, p,	cv::Point2f(p.x, p.y-10), CV_RGB(255,0,0), 2);

        QImage *tmpImg = Mat2QImage(img_complete_clone);
        QPixmap image = QPixmap::fromImage(*tmpImg);
        dialog->labelImage->setPixmap(image);
        delete tmpImg;
    }

}

void SelectPoint::resetPoint() {
    img_complete_clone = img_complete.clone();
    QImage *tmpImg = Mat2QImage(img_complete_clone);
    QPixmap image = QPixmap::fromImage(*tmpImg);
    dialog->labelImage->setPixmap(image);
    delete tmpImg;
    std::cout << "s1: " << this->obj_points.size() << " s2: " << scene_points.size() << std::endl;
    this->obj_points.clear();
    scene_points.clear();
    dialog->labelMsg->setText("Select Minimum four (4) points and then press Compute!: Select a Point from the Object View");
    std::cout << "s1: " << obj_points.size() << " s2: " << scene_points.size() << std::endl;
}

void SelectPoint::resetLastPoint() {
    bool draw = false;
    if(obj_points.size() > scene_points.size()) {
        obj_points.pop_back();
        draw = true;
    } else if (obj_points.size() == scene_points.size()) {
        std::cout << "Entro qui" << std::endl;
        std::cout << obj_points.size() << " " << scene_points.size() << std::endl;
        obj_points.pop_back();
        scene_points.pop_back();
        draw = true;
    }

    if(draw) {
        img_complete_clone = img_complete.clone();
        for(int i = 0; i < obj_points.size(); ++i) {

            cv::line(img_complete_clone, obj_points[i], cv::Point2f(scene_points[i].x + img_obj.cols, scene_points[i].y), CV_RGB(0,0,255), 2);

            cv::line(img_complete_clone, obj_points[i],	cv::Point2f(obj_points[i].x+10, obj_points[i].y), CV_RGB(255,0,0), 2);
            cv::line(img_complete_clone, obj_points[i],	cv::Point2f(obj_points[i].x-10, obj_points[i].y), CV_RGB(255,0,0), 2);
            cv::line(img_complete_clone, obj_points[i],	cv::Point2f(obj_points[i].x, obj_points[i].y+10), CV_RGB(255,0,0), 2);
            cv::line(img_complete_clone, obj_points[i],	cv::Point2f(obj_points[i].x, obj_points[i].y-10), CV_RGB(255,0,0), 2);

            cv::Point2f p(scene_points[i].x + img_obj.cols, scene_points[i].y);

            cv::line(img_complete_clone, p,	cv::Point2f(p.x+10, p.y), CV_RGB(255,0,0), 2);
            cv::line(img_complete_clone, p,	cv::Point2f(p.x-10, p.y), CV_RGB(255,0,0), 2);
            cv::line(img_complete_clone, p,	cv::Point2f(p.x, p.y+10), CV_RGB(255,0,0), 2);
            cv::line(img_complete_clone, p,	cv::Point2f(p.x, p.y-10), CV_RGB(255,0,0), 2);

        }
        QImage *tmpImg = Mat2QImage(img_complete_clone);
        QPixmap image = QPixmap::fromImage(*tmpImg);
        dialog->labelImage->setPixmap(image);
        delete tmpImg;
    }

}

int SelectPoint::getWidth() {
    return img_complete.cols;
}

int SelectPoint::getHeight() {
    return img_complete.rows;
}


void SelectPoint::setNames(std::string objName, std::string sceneName) {
    this->objName = objName;
}

void SelectPoint::compute() {
    if(isPossibleToCompute) {
        h = new Homography(obj_points, scene_points);
        H = h->calcHomography(filename);
        h->draw(img_obj, img_scene, H);
    }
}
